@@ -13,9 +13,7 @@ use Mojo::JSON;
 use Mojo::DOM;
 use Mojo::Loader;
 
-use Mojolicious::Plugin::Vparam::Address;
-
-our $VERSION    = '1.19';
+our $VERSION    = '1.20';
 
 # Shift for convert ASCII char position to simple sequence 0,1,2...9,A,B,C,,,
 our $CHAR_SHIFT = ord('A') - 10;
@@ -76,7 +74,7 @@ L<Mojo::JSON::Pointer>.
 
 =item *
 
-Simple XML values extraction and validation using CSS selector engine
+Simple XML/HTML values extraction and validation using CSS selector engine
 from L<Mojo::DOM::CSS> or XPath from L<XML::LibXML>.
 
 =item *
@@ -273,6 +271,7 @@ Set new type $name if defined %opts. Else return type $name definition.
     $self->vtype('mytype');
 
     # Set type
+    # load  - requires modules
     # pre   - get int
     # valid - check for not empty
     # post  - force number
@@ -430,7 +429,7 @@ sub _load_class($) {
 # Around deprication
 sub _params($$) {
     return @{ $_[0]->every_param( $_[1] ) } if $_[0]->can('every_param');
-    return @{ $_[0]->param( $_[1] ) }       if $_[0]->can('param');
+    return $_[0]->param( $_[1] )            if $_[0]->can('param');
     die 'Looks like Mojo again depricate module Mojo::Controller';
 }
 
@@ -679,10 +678,6 @@ Email adress.
 sub _check_email($) {
     return 'Value not defined'          unless defined $_[0];
     return 'Value is not set'           unless length  $_[0];
-
-    my $e = _load_class('Mail::RFC822::Address');
-    die $e if $e;
-
     return 'Wrong format' unless Mail::RFC822::Address::valid( $_[0] );
     return 0;
 }
@@ -825,7 +820,6 @@ sub _check_isin($) {
 
     my $str = $_[0];
     s{([A-Z])}{(ord($1)-$CHAR_SHIFT)}eg for $str;
-
     my $crc = 0;
     my @str = reverse split '', $str;
     for my $i ( 0 .. $#str  ) {
@@ -834,8 +828,34 @@ sub _check_isin($) {
         $digit -= 9 if $digit > 9;
         $crc += $digit;
     }
+    return 'Checksum error'         if $crc % 10;
 
-    return ($crc % 10 ? 'Checksum error' : 0);
+    return 0;
+}
+
+=head2 barcode
+
+Barcode: EAN-13, EAN-8, EAN 5, EAN 2, UPC-12, ITF-14, JAN, UPC, etc.
+
+    $self->vparam(barcode => 'goods');
+
+=cut
+
+sub _check_barcode($) {
+    return 'Value not defined'      unless defined $_[0];
+    return 'Value not set'          unless length  $_[0];
+    return 'Wrong format'           unless $_[0] =~ m{^[0-9]+$};
+
+    my $crc = 0;
+    my @str = reverse split '', $_[0];
+    for my $i ( 0 .. $#str  ) {
+        my $digit = $str[$i];
+        $digit *= 3 if $i % 2;
+        $crc += $digit;
+    }
+    return 'Checksum error'         if $crc % 10;
+
+    return 0;
 }
 
 =head2 inn
@@ -904,6 +924,11 @@ Default value. Default: undef.
     # Supress myparam to be undefined and error
     $self->vparam(myparam => 'str', default => '');
 
+=head2 load
+
+Autoload module for this type.
+Can be module name, array of module names or sub.
+
 =head2 pre $mojo, &sub
 
 Incoming filter sub. Used for primary filtration: string length and trim, etc.
@@ -1041,7 +1066,7 @@ Arrays always return as arrayref. But undefined values will be skipped.
 =head2 jpath
 
 If you POST data not form but raw JSON you can use JSON Pointer selectors
-to get and validate parameters.
+from L<Mojo::JSON::Pointer> to get and validate parameters.
 
     # POST data contains:
     # {"point":{"address":"some", "lon": 45.123456, "lat": 38.23452}}
@@ -1054,7 +1079,7 @@ to get and validate parameters.
 
 =head2 cpath
 
-Same as jpath but parse XML/HTML using CSS selectors.
+Same as jpath but parse XML/HTML using CSS selectors from L<Mojo::DOM::CSS>.
 
     # POST data contains:
     # <Point>
@@ -1072,7 +1097,7 @@ Same as jpath but parse XML/HTML using CSS selectors.
 
 =head2 xpath
 
-Same as cpath but parse XML/HTML using XPath selectors.
+Same as cpath but parse XML/HTML using XPath selectors from L<XML::LibXML>.
 
     # POST data contains:
     # <Point time="2016-11-25 14:39:00 +0300">
@@ -1252,7 +1277,9 @@ sub _parse_int($) {
 sub _parse_number($) {
     my ($str) = @_;
     return undef unless defined $str;
-    my ($number) = $str =~ m{([-+]?\d+(?:\.\d*)?)};
+    my ($number) = $str =~ m{([-+]?\d+(?:[.,]\d*)?)};
+    return undef unless defined $number;
+    tr{,}{.} for $number;
     return $number;
 }
 
@@ -1261,12 +1288,6 @@ sub _parse_number($) {
 sub _parse_date($;$) {
     my ($str, $tz) = @_;
 
-    my $e1 = _load_class('DateTime');
-    die $e1 if $e1;
-
-    my $e2 = _load_class('DateTime::Format::DateParse');
-    die $e2 if $e2;
-
     return undef unless defined $str;
     s{^\s+}{}, s{\s+$}{} for $str;
     return undef unless length $str;
@@ -1397,6 +1418,13 @@ sub _parse_isin($) {
     return uc $str;
 }
 
+sub _parse_barcode($) {
+    my ($str) = @_;
+    return undef unless defined $str;
+    s{[^0-9]}{}g for $str;
+    return $str;
+}
+
 # Plugin
 sub register {
     my ($self, $app, $conf) = @_;
@@ -1485,6 +1513,7 @@ sub register {
 
         # Date and Time
         date        => {
+            load    => ['DateTime', 'DateTime::Format::DateParse'],
             pre     => sub { _parse_date _trim  $_[1] },
             valid   => sub { _check_date        $_[1] },
             post    => sub {
@@ -1495,6 +1524,7 @@ sub register {
             },
         },
         time        => {
+            load    => ['DateTime', 'DateTime::Format::DateParse'],
             pre     => sub { _parse_date _trim  $_[1] },
             valid   => sub { _check_time        $_[1] },
             post    => sub {
@@ -1505,6 +1535,7 @@ sub register {
             },
         },
         datetime    => {
+            load    => ['DateTime', 'DateTime::Format::DateParse'],
             pre     => sub { _parse_date _trim  $_[1] },
             valid   => sub { _check_datetime    $_[1] },
             post    => sub {
@@ -1523,6 +1554,7 @@ sub register {
 
         # Internet
         email       => {
+            load    => 'Mail::RFC822::Address',
             pre     => sub { _trim              $_[1] },
             valid   => sub { _check_email       $_[1] },
         },
@@ -1547,6 +1579,7 @@ sub register {
             valid   => sub { _check_json        $_[1] },
         },
         address     => {
+            load    => 'Mojolicious::Plugin::Vparam::Address',
             pre     => sub { _parse_address     $_[1] },
             valid   => sub { _check_address     $_[1], $conf->{address_secret}},
         },
@@ -1556,6 +1589,11 @@ sub register {
             pre     => sub { _parse_isin        $_[1] },
             valid   => sub { _check_isin        $_[1] },
         },
+        # Barcode
+        barcode     => {
+            pre     => sub { _parse_barcode     $_[1] },
+            valid   => sub { _check_barcode     $_[1] },
+        },
 
         # RU
         inn         => {
@@ -1723,7 +1761,7 @@ sub register {
             if( exists $attr{skip} ) {
                 if( 'CODE' eq ref $attr{skip} ) {
                     # Skip by sub result
-                    next if $attr{skip}->($self);
+                    next if $attr{skip}->($self, $name);
                 } elsif( $attr{skip} ) {
                     # Skip by flag
                     next;
@@ -1766,6 +1804,21 @@ sub register {
                 }
             }
 
+            # Preload module if required
+            if( my $load = $attr{load} ) {
+                if( 'CODE' eq ref $load ) {
+                    $load->($self, $name);
+                } elsif( 'ARRAY' eq ref $load ) {
+                    for my $module ( @$load ) {
+                        my $e = _load_class( $module );
+                        die $e if $e;
+                    }
+                } else {
+                    my $e = _load_class( $load );
+                    die $e if $e;
+                }
+            }
+
             # Get value
             my @input;
             if( $attr{jpath} ) {
